@@ -21,6 +21,7 @@
 import logging
 import sys
 import copy
+from functools import reduce  # pylint: disable=redefined-builtin
 import time
 import types
 import signal
@@ -38,9 +39,6 @@ from lava_dispatcher.utils.constants import (
 )
 from lava_dispatcher.utils.strings import seconds_to_str
 
-if sys.version > '3':
-    from functools import reduce  # pylint: disable=redefined-builtin
-
 
 class LAVAError(Exception):
     """ Base class for all exceptions in LAVA """
@@ -297,7 +295,7 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
                     new_connection = action.run(connection,
                                                 action_max_end_time, args)
             except LAVAError as exc:
-                action.logger.error(str(exc))
+                action.logger.exception(str(exc))
                 # allows retries without setting errors, which make the job incomplete.
                 failed = True
                 action.results = {'fail': str(exc)}
@@ -354,24 +352,27 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         self.job = None
         self.logger = logging.getLogger('dispatcher')
         self.__results__ = OrderedDict()
-        self.timeout = Timeout(self.name)
+        self.timeout = Timeout(self.name, exception=self.timeout_exception)
         self.max_retries = 1  # unless the strategy or the job parameters change this, do not retry
         self.diagnostics = []
         self.protocols = []  # list of protocol objects supported by this action, full list in job.protocols
-        self.section = None
-        self.connection_timeout = Timeout(self.name)
+        self.connection_timeout = Timeout(self.name, exception=self.timeout_exception)
         self.character_delay = 0
         self.force_prompt = False
 
+    # Section
+    section = None
     # public actions (i.e. those who can be referenced from a job file) must
     # declare a 'class-type' name so they can be looked up.
     # summary and description are used to identify instances.
     name = None
+    # Used in the pipeline to explain what the commands will attempt to do.
+    description = None
     # A short summary of this instance of a class inheriting from Action.  May
     # be None.
     summary = None
-    # Used in the pipeline to explain what the commands will attempt to do.
-    description = None
+    # Exception to raise when this action is timing out
+    timeout_exception = JobError
 
     @property
     def data(self):
@@ -435,8 +436,6 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         except ValueError:
             raise LAVABug("Action parameters need to be a dictionary")
 
-        # Set the timeout name now
-        self.timeout.name = self.name
         # Overide the duration if needed
         if 'timeout' in self.parameters:
             # preserve existing overrides
@@ -516,7 +515,7 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         """
         pass
 
-    def run_command(self, command_list, allow_silent=False, allow_fail=False):  # pylint: disable=too-many-branches
+    def run_command(self, command_list, allow_silent=False, allow_fail=False, cwd=None):  # pylint: disable=too-many-branches
         """
         Single location for all external command operations on the
         dispatcher, without using a shell and with full structured logging.
@@ -545,33 +544,23 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         command_list = ['nice'] + [str(s) for s in command_list]
         self.logger.debug("%s", ' '.join(command_list))
         try:
-            log = subprocess.check_output(command_list, stderr=subprocess.STDOUT)
-            log = log.decode('utf-8')  # pylint: disable=redefined-variable-type
+            log = subprocess.check_output(command_list, stderr=subprocess.STDOUT,
+                                          cwd=cwd)
+            log = log.decode('utf-8', errors="replace")  # pylint: disable=redefined-variable-type
         except subprocess.CalledProcessError as exc:
             # the errors property doesn't support removing errors
             errors = []
-            if sys.version > '3':
-                if exc.output:
-                    errors.append(exc.output.strip().decode('utf-8'))
-                else:
-                    errors.append(str(exc))
-                msg = '[%s] command %s\nmessage %s\noutput %s\n' % (
-                    self.name, [i.strip() for i in exc.cmd], str(exc), str(exc).split('\n'))
+            if exc.output:
+                errors.append(exc.output.strip().decode('utf-8', errors="replace"))
             else:
-                if exc.output:
-                    errors.append(exc.output.strip())
-                elif exc.message:
-                    errors.append(exc.message)
-                else:
-                    errors.append(str(exc))
-                msg = "[%s] command %s\nmessage %s\noutput %s\nexit code %s" % (
-                    self.name, [i.strip() for i in exc.cmd], [i.strip() for i in exc.message],
-                    exc.output.split('\n'), exc.returncode)
+                errors.append(str(exc))
+            msg = '[%s] command %s\nmessage %s\noutput %s\n' % (
+                self.name, [i.strip() for i in exc.cmd], str(exc), str(exc).split('\n'))
 
             # the exception is raised due to a non-zero exc.returncode
             if allow_fail:
                 self.logger.info(msg)
-                log = exc.output.strip()
+                log = exc.output.strip().decode('utf-8', errors="replace")
             else:
                 for error in errors:
                     self.errors = error
@@ -583,7 +572,8 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         if not log and allow_silent:
             return self.errors == []
         else:
-            self.logger.debug('command output %s', log)
+            for line in log.split("\n"):
+                self.logger.debug('output: %s', line)
             return log
 
     def call_protocols(self):
@@ -780,7 +770,7 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
             return
         if not isinstance(timeout, dict):
             raise JobError("Invalid timeout %s" % str(timeout))
-        self.timeout = Timeout(self.name, Timeout.parse(timeout))
+        self.timeout.duration = Timeout.parse(timeout)
 
     def _override_connection_timeout(self, timeout):
         """
@@ -790,7 +780,7 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
             return
         if not isinstance(timeout, dict):
             raise JobError("Invalid connection timeout %s" % str(timeout))
-        self.connection_timeout = Timeout(self.name, Timeout.parse(timeout))
+        self.connection_timeout.duration = Timeout.parse(timeout)
 
     def log_action_results(self, fail=False):
         if self.results and isinstance(self.logger, YAMLLogger):
@@ -818,33 +808,30 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
 
     @nottest
     def test_needs_deployment(self, parameters):  # pylint: disable=no-self-use
-        needs_deployment = False
         if parameters['namespace'] in parameters['test_info']:
             testclasses = parameters['test_info'][parameters['namespace']]
             for testclass in testclasses:
                 if testclass['class'].needs_deployment_data():
-                    needs_deployment = True
-        return needs_deployment
+                    return True
+        return False
 
     @nottest
     def test_has_shell(self, parameters):  # pylint: disable=no-self-use
-        has_shell = False
         if parameters['namespace'] in parameters['test_info']:
             testclasses = parameters['test_info'][parameters['namespace']]
             for testclass in testclasses:
                 if testclass['class'].has_shell():
-                    has_shell = True
-        return has_shell
+                    return True
+        return False
 
     @nottest
     def test_needs_overlay(self, parameters):  # pylint: disable=no-self-use
-        needs_overlay = False
         if parameters['namespace'] in parameters['test_info']:
             testclasses = parameters['test_info'][parameters['namespace']]
             for testclass in testclasses:
                 if testclass['class'].needs_overlay():
-                    needs_overlay = True
-        return needs_overlay
+                    return True
+        return False
 
 
 class Timeout(object):
@@ -859,12 +846,12 @@ class Timeout(object):
     If a connection is set, this timeout is used per pexpect operation on that connection.
     If a connection is not set, this timeout applies for the entire run function of the action.
     """
-    def __init__(self, name, duration=ACTION_TIMEOUT, protected=False):
+    def __init__(self, name, duration=ACTION_TIMEOUT, exception=JobError):
         self.name = name
         self.start = 0
         self.elapsed_time = -1
         self.duration = duration  # Actions can set timeouts higher than the clamp.
-        self.protected = protected
+        self.exception = exception
 
     @classmethod
     def default_duration(cls):
@@ -888,7 +875,7 @@ class Timeout(object):
 
     def _timed_out(self, signum, frame):  # pylint: disable=unused-argument
         duration = int(time.time() - self.start)
-        raise JobError("%s timed out after %s seconds" % (self.name, duration))
+        raise self.exception("%s timed out after %s seconds" % (self.name, duration))
 
     @contextmanager
     def __call__(self, action_max_end_time=None):
@@ -906,6 +893,9 @@ class Timeout(object):
             # If duration is lower than 0, then the timeout should be raised now.
             # Calling signal.alarm in this case will only deactivate the alarm
             # (by passing 0 or the unsigned value).
+            # Deactivate any previous alarm and set elapse_time prior to raise
+            signal.alarm(0)
+            self.elapsed_time = 0
             self._timed_out(None, None)
 
         signal.signal(signal.SIGALRM, self._timed_out)
@@ -913,18 +903,7 @@ class Timeout(object):
 
         try:
             yield max_end_time
-        except:
-            raise
         finally:
             # clear the timeout alarm, the action has returned
             signal.alarm(0)
             self.elapsed_time = time.time() - self.start
-
-    def modify(self, duration):
-        """
-        Called from the parser if the job YAML wants to set an override on a per-action
-        timeout. Complete job timeouts can be larger than the clamp.
-        """
-        if self.protected:
-            raise JobError("Trying to modify a protected timeout: %s.", self.name)
-        self.duration = max(min(OVERRIDE_CLAMP_DURATION, duration), 1)  # FIXME: needs support in /etc/
