@@ -45,7 +45,7 @@ from lava_dispatcher.utils.filesystem import (
     copy_in_overlay,
     copy_overlay_to_sparse_fs,
 )
-from lava_dispatcher.utils.shell import infrastructure_error
+from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.compression import (
     compress_file,
     decompress_file,
@@ -60,19 +60,17 @@ from lava_dispatcher.actions.deploy.prepare import PrepareKernelAction
 
 class ApplyOverlayGuest(Action):
 
+    name = "apply-overlay-guest"
+    description = "prepare a qcow2 drive containing the overlay"
+    summary = "build a guest filesystem with the overlay"
+
     def __init__(self):
         super(ApplyOverlayGuest, self).__init__()
-        self.name = "apply-overlay-guest"
-        self.summary = "build a guest filesystem with the overlay"
-        self.description = "prepare a qcow2 drive containing the overlay"
         self.guest_filename = 'lava-guest.qcow2'
 
     def validate(self):
         super(ApplyOverlayGuest, self).validate()
         self.set_namespace_data(action=self.name, label='guest', key='name', value=self.guest_filename)
-        lava_test_results_base = self.parameters['deployment_data']['lava_test_results_dir']
-        lava_test_results_dir = lava_test_results_base % self.job.job_id
-        self.set_namespace_data(action='test', label='results', key='lava_test_results_dir', value=lava_test_results_dir)
         if 'guest' not in self.job.device['actions']['deploy']['methods']['image']['parameters']:
             self.errors = "Device configuration does not specify size of guest filesystem."
 
@@ -94,22 +92,30 @@ class ApplyOverlayGuest(Action):
 
 class ApplyOverlayImage(Action):
 
-    def __init__(self):
+    name = "apply-overlay-image"
+    description = "apply overlay via guestfs to the test image"
+    summary = "apply overlay to test image"
+
+    def __init__(self, image_key='image', use_root_partition=True):
         super(ApplyOverlayImage, self).__init__()
-        self.name = "apply-overlay-image"
-        self.summary = "apply overlay to test image"
-        self.description = "apply overlay via guestfs to the test image"
+        self.image_key = image_key
+        self.use_root_partition = use_root_partition
 
     def run(self, connection, max_end_time, args=None):
         overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
         if overlay_file:
             self.logger.debug("Overlay: %s", overlay_file)
-            decompressed_image = self.get_namespace_data(action='download-action', label='image', key='file')
+            decompressed_image = self.get_namespace_data(action='download-action',
+                                                         label=self.image_key, key='file')
             self.logger.debug("Image: %s", decompressed_image)
-            root_partition = self.parameters['image'].get('root_partition')
-            if root_partition is None:
-                raise JobError("Unable to apply the overlay image without 'root_partition'")
-            self.logger.debug("root_partition: %s", root_partition)
+            root_partition = None
+
+            if self.use_root_partition:
+                root_partition = self.parameters[self.image_key].get('root_partition')
+                if root_partition is None:
+                    raise JobError("Unable to apply the overlay image without 'root_partition'")
+                self.logger.debug("root_partition: %s", root_partition)
+
             copy_in_overlay(decompressed_image, root_partition, overlay_file)
         else:
             self.logger.debug("No overlay to deploy")
@@ -118,19 +124,20 @@ class ApplyOverlayImage(Action):
 
 class ApplyOverlaySparseImage(Action):
 
+    name = "apply-overlay-sparse-image"
+    description = "apply overlay to sparse image"
+    summary = "apply overlay to sparse image"
+
     def __init__(self, image_key):
         super(ApplyOverlaySparseImage, self).__init__()
-        self.name = "apply-overlay-sparse-image"
-        self.summary = "apply overlay to sparse image"
-        self.description = "apply overlay to sparse image"
         self.image_key = image_key  # the sparse image key in the parameters
 
     def validate(self):
         super(ApplyOverlaySparseImage, self).validate()
-        self.errors = infrastructure_error('/usr/bin/simg2img')
-        self.errors = infrastructure_error('/bin/mount')
-        self.errors = infrastructure_error('/bin/umount')
-        self.errors = infrastructure_error('/usr/bin/img2simg')
+        which('simg2img')
+        which('mount')
+        which('umount')
+        which('img2simg')
 
     def run(self, connection, max_end_time, args=None):
         overlay_file = self.get_namespace_data(action='compress-overlay',
@@ -150,11 +157,10 @@ class PrepareOverlayTftp(Action):
     """
     Extracts the ramdisk or nfsrootfs in preparation for the lava overlay
     """
-    def __init__(self):
-        super(PrepareOverlayTftp, self).__init__()
-        self.name = "prepare-tftp-overlay"
-        self.summary = "extract ramdisk or nfsrootfs"
-        self.description = "extract ramdisk or nfsrootfs in preparation for lava overlay"
+
+    name = "prepare-tftp-overlay"
+    description = "extract ramdisk or nfsrootfs in preparation for lava overlay"
+    summary = "extract ramdisk or nfsrootfs"
 
     def populate(self, parameters):
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
@@ -167,6 +173,8 @@ class PrepareOverlayTftp(Action):
             self.internal_pipeline.add_action(PrepareKernelAction())
         self.internal_pipeline.add_action(ConfigurePreseedFile())  # idempotent, checks for a preseed parameter
         self.internal_pipeline.add_action(CompressRamdisk())  # idempotent, checks for a ramdisk parameter
+        if 'depthcharge' in self.job.device['actions']['boot']['methods']:
+            self.internal_pipeline.add_action(PrepareKernelAction())
 
     def run(self, connection, max_end_time, args=None):
         connection = super(PrepareOverlayTftp, self).run(connection, max_end_time, args)
@@ -188,11 +196,10 @@ class ApplyOverlayTftp(Action):
     Other actions applying overlays for other deployments need their
     own logic.
     """
-    def __init__(self):
-        super(ApplyOverlayTftp, self).__init__()
-        self.name = "apply-overlay-tftp"
-        self.summary = "apply lava overlay test files"
-        self.description = "unpack the overlay into the nfsrootfs or ramdisk"
+
+    name = "apply-overlay-tftp"
+    description = "unpack the overlay into the nfsrootfs or ramdisk"
+    summary = "apply lava overlay test files"
 
     def validate(self):
         super(ApplyOverlayTftp, self).validate()
@@ -279,11 +286,13 @@ class ExtractRootfs(Action):  # pylint: disable=too-many-instance-attributes
     """
     Unpacks the rootfs and applies the overlay to it
     """
+
+    name = "extract-rootfs"
+    description = "unpack rootfs"
+    summary = "unpack rootfs, ready to apply lava overlay"
+
     def __init__(self):
         super(ExtractRootfs, self).__init__()
-        self.name = "extract-rootfs"
-        self.description = "unpack rootfs"
-        self.summary = "unpack rootfs, ready to apply lava overlay"
         self.param_key = 'rootfs'
         self.file_key = "root"
         self.extra_compression = ['xz']
@@ -306,11 +315,13 @@ class ExtractNfsRootfs(ExtractRootfs):
     """
     Unpacks the nfsrootfs and applies the overlay to it
     """
+
+    name = "extract-nfsrootfs"
+    description = "unpack nfsrootfs"
+    summary = "unpack nfsrootfs, ready to apply lava overlay"
+
     def __init__(self):
         super(ExtractNfsRootfs, self).__init__()
-        self.name = "extract-nfsrootfs"
-        self.description = "unpack nfsrootfs"
-        self.summary = "unpack nfsrootfs, ready to apply lava overlay"
         self.param_key = 'nfsrootfs'
         self.file_key = "nfsroot"
 
@@ -358,11 +369,10 @@ class ExtractModules(Action):
     If modules are specified in the deploy parameters, unpack the modules
     whilst the nfsrootfs or ramdisk are unpacked.
     """
-    def __init__(self):
-        super(ExtractModules, self).__init__()
-        self.name = "extract-modules"
-        self.summary = "extract kernel modules"
-        self.description = "extract supplied kernel modules"
+
+    name = "extract-modules"
+    description = "extract supplied kernel modules"
+    summary = "extract kernel modules"
 
     def run(self, connection, max_end_time, args=None):
         if not self.parameters.get('modules', None):  # idempotency
@@ -409,11 +419,13 @@ class ExtractRamdisk(Action):
     for other actions to modify. Needs CompressRamdisk to
     recreate the ramdisk with modifications.
     """
+
+    name = "extract-overlay-ramdisk"
+    description = "extract ramdisk to a temporary directory"
+    summary = "extract the ramdisk"
+
     def __init__(self):
         super(ExtractRamdisk, self).__init__()
-        self.name = "extract-overlay-ramdisk"
-        self.summary = "extract the ramdisk"
-        self.description = "extract ramdisk to a temporary directory"
         self.skip = False
 
     def validate(self):
@@ -450,7 +462,7 @@ class ExtractRamdisk(Action):
                 ramdisk, ramdisk_compressed_data, UBOOT_DEFAULT_HEADER_LENGTH)).split(' ')
             try:
                 self.run_command(cmd)
-            except:
+            except Exception:
                 raise LAVABug('Unable to remove uboot header: %s' % ramdisk)
         else:
             # give the file a predictable name
@@ -472,11 +484,13 @@ class CompressRamdisk(Action):
     """
      recreate ramdisk, with overlay in place
     """
+
+    name = "compress-ramdisk"
+    description = "recreate a ramdisk with the overlay applied."
+    summary = "compress ramdisk with overlay"
+
     def __init__(self):
         super(CompressRamdisk, self).__init__()
-        self.name = "compress-ramdisk"
-        self.summary = "compress ramdisk with overlay"
-        self.description = "recreate a ramdisk with the overlay applied."
         self.mkimage_arch = None
         self.add_header = None
         self.skip = False
@@ -493,7 +507,7 @@ class CompressRamdisk(Action):
             self.add_header = self.job.device['actions']['deploy']['parameters'].get('add_header', None)
             if self.add_header is not None:
                 if self.add_header == 'u-boot':
-                    self.errors = infrastructure_error('mkimage')
+                    which('mkimage')
                     if 'mkimage_arch' not in self.job.device['actions']['deploy']['parameters']:
                         self.errors = "Missing architecture for uboot mkimage support (mkimage_arch in deploy parameters)"
                         return
@@ -555,9 +569,12 @@ class CompressRamdisk(Action):
                 raise InfrastructureError("Unable to add uboot header to ramdisk")
             final_file = ramdisk_uboot
 
-        shutil.move(final_file, os.path.join(tftp_dir, os.path.basename(final_file)))
-        self.logger.debug("rename %s to %s",
-                          final_file, os.path.join(tftp_dir, os.path.basename(final_file)))
+        full_path = os.path.join(tftp_dir, os.path.basename(final_file))
+        shutil.move(final_file, full_path)
+        self.logger.debug("rename %s to %s", final_file, full_path)
+        self.set_namespace_data(
+            action=self.name, label='file', key='full-path', value=full_path)
+
         if self.parameters['to'] == 'tftp':
             suffix = self.get_namespace_data(action='tftp-deploy', label='tftp', key='suffix')
             if not suffix:
@@ -574,18 +591,19 @@ class CompressRamdisk(Action):
 
 class ApplyLxcOverlay(Action):
 
+    name = "apply-lxc-overlay"
+    description = "apply the overlay to the container by copying"
+    summary = "apply overlay on the container"
+
     def __init__(self):
         super(ApplyLxcOverlay, self).__init__()
-        self.name = "apply-lxc-overlay"
-        self.summary = "apply overlay on the container"
-        self.description = "apply the overlay to the container by copying"
         self.lava_test_dir = os.path.realpath(
             '%s/../../lava_test_shell' % os.path.dirname(__file__))
         self.scripts_to_copy = ['lava-test-runner']
 
     def validate(self):
         super(ApplyLxcOverlay, self).validate()
-        self.errors = infrastructure_error('tar')
+        which('tar')
         if not os.path.exists(self.lava_test_dir):
             self.errors = "Missing lava-test-runner: %s" % self.lava_test_dir
 
@@ -625,11 +643,10 @@ class ApplyLxcOverlay(Action):
 
 
 class ConfigurePreseedFile(Action):
-    def __init__(self):
-        super(ConfigurePreseedFile, self).__init__()
-        self.name = "configure-preseed-file"
-        self.summary = "add commands to installer config"
-        self.description = "add commands to automated installers, to copy the lava test overlay to the installed system"
+
+    name = "configure-preseed-file"
+    description = "add commands to automated installers, to copy the lava test overlay to the installed system"
+    summary = "add commands to installer config"
 
     def run(self, connection, max_end_time, args=None):
         if 'deployment_data' not in self.parameters:
