@@ -38,6 +38,7 @@ import zlib
 from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.protocols.lxc import LxcProtocol
 from lava_dispatcher.actions.deploy import DeployAction
+from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.connections.serial import ConnectDevice
 from lava_dispatcher.action import (
     Action,
@@ -54,6 +55,7 @@ from lava_dispatcher.utils.compression import untar_file
 from lava_dispatcher.utils.filesystem import (
     copy_to_lxc,
     lava_lxc_home,
+    copy_overlay_to_lxc,
 )
 from lava_dispatcher.utils.constants import (
     FILE_DOWNLOAD_CHUNK_SIZE,
@@ -63,10 +65,7 @@ from lava_dispatcher.utils.constants import (
 from lava_dispatcher.actions.boot.fastboot import EnterFastbootAction
 from lava_dispatcher.actions.boot.u_boot import UBootEnterFastbootAction
 
-if sys.version_info[0] == 2:
-    import urlparse as lavaurl
-elif sys.version_info[0] == 3:
-    import urllib.parse as lavaurl  # pylint: disable=no-name-in-module,import-error
+import urllib.parse as lavaurl
 
 # pylint: disable=logging-not-lazy
 
@@ -80,11 +79,13 @@ class DownloaderAction(RetryAction):
     The retry pipeline for downloads.
     To allow any deploy action to work with multinode, each call *must* set a unique path.
     """
+
+    name = "download-retry"
+    description = "download with retry"
+    summary = "download-retry"
+
     def __init__(self, key, path, uniquify=True):
         super(DownloaderAction, self).__init__()
-        self.name = "download-retry"
-        self.description = "download with retry"
-        self.summary = "download-retry"
         self.max_retries = 3
         self.key = key  # the key in the parameters of what to download
         self.path = path  # where to download
@@ -95,9 +96,13 @@ class DownloaderAction(RetryAction):
 
         # Find the right action according to the url
         if 'images' in parameters and self.key in parameters['images']:
-            url = lavaurl.urlparse(parameters['images'][self.key]['url'])
+            url = parameters['images'][self.key].get('url')
         else:
-            url = lavaurl.urlparse(parameters[self.key]['url'])
+            url = parameters[self.key].get('url')
+        if url is None:
+            raise JobError("Invalid deploy action: 'url' is missing for '%s'" % self.key)
+
+        url = lavaurl.urlparse(url)
         if url.scheme == 'scp':
             action = ScpDownloadAction(self.key, self.path, url, self.uniquify)
         elif url.scheme == 'http' or url.scheme == 'https':
@@ -122,12 +127,12 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
     as possible.
     """
 
+    name = "download-action"
+    description = "download action"
+    summary = "download-action"
+
     def __init__(self, key, path, url, uniquify=True):
         super(DownloadHandler, self).__init__()
-        self.name = "download-action"
-        self.description = "download action"
-        self.summary = "download-action"
-
         self.url = url
         self.key = key
         # If uniquify is True, store the files in a sub-directory to keep the
@@ -157,64 +162,6 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             return (os.path.join(path, '.'.join(parts[:-1])),
                     parts[-1])
 
-    @contextlib.contextmanager
-    def _decompressor_stream(self):  # pylint: disable=too-many-branches
-        dwnld_file = None
-        compression = False
-        if 'images' in self.parameters and self.key in self.parameters['images']:
-            compression = self.parameters['images'][self.key].get('compression', False)
-        else:
-            if self.key == 'ramdisk':
-                self.logger.debug("Not decompressing ramdisk as can be used compressed.")
-            else:
-                compression = self.parameters[self.key].get('compression', False)
-
-        fname, _ = self._url_to_fname_suffix(self.path, compression)
-        if os.path.isdir(fname):
-            raise JobError("Download '%s' is a directory, not a file" % fname)
-        if os.path.exists(fname):
-            os.remove(fname)
-
-        decompressor = None
-        if compression:
-            if compression == 'gz':
-                decompressor = zlib.decompressobj(16 + zlib.MAX_WBITS)
-            elif compression == 'bz2':
-                decompressor = bz2.BZ2Decompressor()
-            elif compression == 'xz':
-                decompressor = lzma.LZMADecompressor()  # pylint: disable=no-member
-            self.logger.debug("Using %s decompression" % compression)
-        else:
-            self.logger.debug("No compression specified.")
-
-        def write(buff):
-            if decompressor:
-                try:
-                    buff = decompressor.decompress(buff)
-                except EOFError as eof_exc:
-                    # EOFError can be raised when decompressing a bz2 archive
-                    # generated by pbzip2. If there is something in unsused_data
-                    # try to continue decompression.
-                    if compression == 'bz2' and decompressor.unused_data:
-                        buff = decompressor.unused_data
-                    else:
-                        error_message = str(eof_exc)
-                        self.logger.exception(error_message)
-                        raise JobError(error_message)
-                except (IOError, lzma.error, zlib.error) as exc:  # pylint: disable=no-member
-                    error_message = str(exc)
-                    self.logger.exception(error_message)
-                    raise JobError(error_message)
-            dwnld_file.write(buff)
-
-        try:
-            with open(fname, 'wb') as dwnld_file:
-                yield (write, fname)
-        except (IOError, OSError) as exc:
-            msg = "Unable to open %s: %s" % (fname, exc.strerror)
-            self.logger.error(msg)
-            raise InfrastructureError(msg)
-
     def validate(self):
         super(DownloadHandler, self).validate()
         if 'images' in self.parameters and self.key in self.parameters['images']:
@@ -229,6 +176,8 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                                     key='file', value=image_name)
             self.set_namespace_data(action='download-action', label=self.key,
                                     key='image_arg', value=image_arg)
+            self.set_namespace_data(action='download-action', label=self.key,
+                                    key='compression', value=compression)
         else:
             self.url = lavaurl.urlparse(self.parameters[self.key]['url'])
             compression = self.parameters[self.key].get('compression', None)
@@ -236,6 +185,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             overlay = self.parameters.get('overlay', False)
             fname, _ = self._url_to_fname_suffix(self.path, compression)
             self.set_namespace_data(action='download-action', label=self.key, key='file', value=fname)
+            self.set_namespace_data(action='download-action', label=self.key, key='compression', value=compression)
 
         if overlay:
             self.set_namespace_data(action='download-action', label=self.key, key='overlay', value=overlay)
@@ -278,47 +228,105 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             if exc.errno != errno.EEXIST:
                 raise InfrastructureError("Unable to create %s: %s" % (self.path, str(exc)))
 
-        # Download the file
-        with self._decompressor_stream() as (writer, fname):
+        if 'images' in self.parameters and self.key in self.parameters['images']:
+            remote = self.parameters['images'][self.key]
+            compression = self.parameters['images'][self.key].get(
+                'compression', False)
 
-            if 'images' in self.parameters and self.key in self.parameters['images']:
-                remote = self.parameters['images'][self.key]
+        else:
+            remote = self.parameters[self.key]
+            if self.key == 'ramdisk':
+                compression = False
+                self.logger.debug(
+                    "Not decompressing ramdisk as can be used compressed.")
             else:
-                remote = self.parameters[self.key]
-            md5sum = remote.get('md5sum', None)
-            sha256sum = remote.get('sha256sum', None)
+                compression = self.parameters[self.key].get('compression',
+                                                            False)
 
-            self.logger.info("downloading %s", remote['url'])
-            self.logger.debug("saving as %s", fname)
+        md5sum = remote.get('md5sum', None)
+        sha256sum = remote.get('sha256sum', None)
 
-            downloaded_size = 0
-            beginning = time.time()
-            # Choose the progress bar (is the size known?)
-            if self.size == -1:
-                self.logger.debug("total size: unknown")
-                last_value = -25 * 1024 * 1024
-                progress = progress_unknown_total
+        fname, _ = self._url_to_fname_suffix(self.path, compression)
+        if os.path.isdir(fname):
+            raise JobError("Download '%s' is a directory, not a file" % fname)
+        if os.path.exists(fname):
+            os.remove(fname)
+
+        self.logger.info("downloading %s", remote['url'])
+        self.logger.debug("saving as %s", fname)
+
+        downloaded_size = 0
+        beginning = time.time()
+        # Choose the progress bar (is the size known?)
+        if self.size == -1:
+            self.logger.debug("total size: unknown")
+            last_value = -25 * 1024 * 1024
+            progress = progress_unknown_total
+        else:
+            self.logger.debug("total size: %d (%dMB)" % (self.size, int(self.size / (1024 * 1024))))
+            last_value = -5
+            progress = progress_known_total
+
+        decompress_command = None
+        if compression:
+            if compression == 'gz':
+                decompress_command = 'gunzip'
+            elif compression == 'bz2':
+                decompress_command = 'bunzip2'
+            elif compression == 'zip':
+                decompress_command = 'unzip'
             else:
-                self.logger.debug("total size: %d (%dMB)" % (self.size, int(self.size / (1024 * 1024))))
-                last_value = -5
-                progress = progress_known_total
+                decompress_command = 'unxz'
+            self.logger.debug("Using %s decompression" % compression)
+        else:
+            self.logger.debug("No compression specified.")
 
-            # Download the file and log the progresses
-            for buff in self.reader():
-                downloaded_size += len(buff)
-                (printing, new_value, msg) = progress(downloaded_size, last_value)
-                if printing:
-                    last_value = new_value
-                    self.logger.debug(msg)
-                md5.update(buff)
-                sha256.update(buff)
-                writer(buff)
+        def update_progress():
+            nonlocal downloaded_size, last_value, md5, sha256
+            downloaded_size += len(buff)
+            (printing, new_value, msg) = progress(downloaded_size,
+                                                  last_value)
+            if printing:
+                last_value = new_value
+                self.logger.debug(msg)
+            md5.update(buff)
+            sha256.update(buff)
 
-            # Log the download speed
-            ending = time.time()
-            self.logger.info("%dMB downloaded in %0.2fs (%0.2fMB/s)" %
-                             (downloaded_size / (1024 * 1024), round(ending - beginning, 2),
-                              round(downloaded_size / (1024 * 1024 * (ending - beginning)), 2)))
+        if compression:
+            try:
+                with open(fname, 'wb') as dwnld_file:
+                    proc = subprocess.Popen([decompress_command],
+                                            stdin=subprocess.PIPE,
+                                            stdout=dwnld_file)
+            except (IOError, OSError) as exc:
+                msg = "Unable to open %s: %s" % (fname, exc.strerror)
+                self.logger.error(msg)
+                raise InfrastructureError(msg)
+
+            with proc.stdin as pipe:
+                for buff in self.reader():
+                    update_progress()
+                    try:
+                        pipe.write(buff)
+                    except BrokenPipeError as exc:
+                        error_message = str(exc)
+                        self.logger.exception(error_message)
+                        msg = "Make sure the 'compression' is corresponding " \
+                              "to the image file type."
+                        self.logger.error(msg)
+                        raise JobError(error_message)
+            proc.wait()
+        else:
+            with open(fname, 'wb') as dwnld_file:
+                for buff in self.reader():
+                    update_progress()
+                    dwnld_file.write(buff)
+
+        # Log the download speed
+        ending = time.time()
+        self.logger.info("%dMB downloaded in %0.2fs (%0.2fMB/s)" %
+                         (downloaded_size / (1024 * 1024), round(ending - beginning, 2),
+                          round(downloaded_size / (1024 * 1024 * (ending - beginning)), 2)))
 
         # set the dynamic data into the context
         self.set_namespace_data(action='download-action', label=self.key, key='file', value=fname)
@@ -401,11 +409,9 @@ class FileDownloadAction(DownloadHandler):
     Download a resource from file (copy)
     """
 
-    def __init__(self, *args, **kwargs):
-        super(FileDownloadAction, self).__init__(*args, **kwargs)
-        self.name = "file-download"
-        self.description = "copy a local file"
-        self.summary = "local file copy"
+    name = "file-download"
+    description = "copy a local file"
+    summary = "local file copy"
 
     def validate(self):
         super(FileDownloadAction, self).validate()
@@ -434,11 +440,9 @@ class HttpDownloadAction(DownloadHandler):
     Download a resource over http or https using requests module
     """
 
-    def __init__(self, *args, **kwargs):
-        super(HttpDownloadAction, self).__init__(*args, **kwargs)
-        self.name = "http-download"
-        self.description = "use http to download the file"
-        self.summary = "http download"
+    name = "http-download"
+    description = "use http to download the file"
+    summary = "http download"
 
     def validate(self):
         super(HttpDownloadAction, self).validate()
@@ -453,7 +457,7 @@ class HttpDownloadAction(DownloadHandler):
                 res = requests.get(
                     self.url.geturl(), allow_redirects=True, stream=True)
                 if res.status_code != requests.codes.OK:  # pylint: disable=no-member
-                    self.errors = "Resources not available at '%s'" % (self.url.geturl())
+                    self.errors = "Resource unavailable at '%s' (%d)" % (self.url.geturl(), res.status_code)
 
             self.size = int(res.headers.get('content-length', -1))
         except requests.Timeout:
@@ -489,11 +493,9 @@ class ScpDownloadAction(DownloadHandler):
     Download a resource over scp
     """
 
-    def __init__(self, *args, **kwargs):
-        super(ScpDownloadAction, self).__init__(*args, **kwargs)
-        self.name = "scp-download"
-        self.description = "Use scp to copy the file"
-        self.summary = "scp download"
+    name = "scp-download"
+    description = "Use scp to copy the file"
+    summary = "scp download"
 
     def validate(self):
         super(ScpDownloadAction, self).validate()
@@ -534,11 +536,12 @@ class LxcDownloadAction(Action):
     Map an already downloaded resource to the correct path.
     """
 
+    name = "lxc-download"
+    description = "Map to the correct lxc path"
+    summary = "lxc download"
+
     def __init__(self, key, path, url):
         super(LxcDownloadAction, self).__init__()
-        self.name = "lxc-download"
-        self.description = "Map to the correct lxc path"
-        self.summary = "lxc download"
         self.key = key
         self.path = path
         self.url = url
@@ -580,11 +583,12 @@ class QCowConversionAction(Action):
     on filename suffix
     """
 
+    name = "qcow2-convert"
+    description = "convert qcow image using qemu-img"
+    summary = "qcow conversion"
+
     def __init__(self, key):
         super(QCowConversionAction, self).__init__()
-        self.name = "qcow2-convert"
-        self.description = "convert qcow image using qemu-img"
-        self.summary = "qcow conversion"
         self.key = key
 
     def run(self, connection, max_end_time, args=None):
@@ -642,11 +646,12 @@ class Download(Deployment):
 
 class DownloadAction(DeployAction):  # pylint:disable=too-many-instance-attributes
 
+    name = "download-deploy"
+    description = "download files and copy to LXC if available"
+    summary = "download deployment"
+
     def __init__(self):
         super(DownloadAction, self).__init__()
-        self.name = "download-deploy"
-        self.description = "download files and copy to LXC if available"
-        self.summary = "download deployment"
         self.download_dir = None
 
     def validate(self):
@@ -668,7 +673,7 @@ class DownloadAction(DeployAction):  # pylint:disable=too-many-instance-attribut
         if self.job.device.get('fastboot_via_uboot', False):
             self.internal_pipeline.add_action(ConnectDevice())
             self.internal_pipeline.add_action(UBootEnterFastbootAction())
-        elif self.job.device.power_command:
+        elif self.job.device.hard_reset_command:
             self.force_prompt = True
             self.internal_pipeline.add_action(ConnectDevice())
             self.internal_pipeline.add_action(ResetDevice())
@@ -681,6 +686,8 @@ class DownloadAction(DeployAction):  # pylint:disable=too-many-instance-attribut
             if image != 'yaml_line':
                 self.internal_pipeline.add_action(DownloaderAction(
                     image, self.download_dir))
+        if self.test_needs_overlay(parameters):
+            self.internal_pipeline.add_action(OverlayAction())
         self.internal_pipeline.add_action(CopyToLxcAction())
 
 
@@ -689,11 +696,12 @@ class CopyToLxcAction(DeployAction):
     Copy downloaded files to LXC within LAVA_LXC_HOME.
     """
 
+    name = "copy-to-lxc"
+    description = "copy files to lxc"
+    summary = "copy to lxc"
+
     def __init__(self):
         super(CopyToLxcAction, self).__init__()
-        self.name = "copy-to-lxc"
-        self.description = "copy files to lxc"
-        self.summary = "copy to lxc"
         self.retries = 3
         self.sleep = 10
 
@@ -722,4 +730,11 @@ class CopyToLxcAction(DeployAction):
             if not src:
                 continue
             copy_to_lxc(lxc_name, src, self.job.parameters['dispatcher'])
+        overlay_file = self.get_namespace_data(action='compress-overlay',
+                                               label='output', key='file')
+        if overlay_file is None:
+            self.logger.debug("skipped %s", self.name)
+        else:
+            copy_overlay_to_lxc(lxc_name, overlay_file,
+                                self.job.parameters['dispatcher'], namespace)
         return connection
