@@ -34,27 +34,33 @@ from lava_dispatcher.actions.deploy.testdef import (
     TestDefinitionAction,
     get_test_action_namespaces,
 )
+from lava_dispatcher.logical import Deployment
 from lava_dispatcher.utils.contextmanager import chdir
 from lava_dispatcher.utils.filesystem import check_ssh_identity_file
-from lava_dispatcher.utils.shell import infrastructure_error
+from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.network import rpcinfo_nfs
 from lava_dispatcher.protocols.multinode import MultinodeProtocol
 from lava_dispatcher.protocols.vland import VlandProtocol
 
 
-class CustomisationAction(DeployAction):
+class Overlay(Deployment):
+    compatibility = 4
+    name = "overlay"
 
-    def __init__(self):
-        super(CustomisationAction, self).__init__()
-        self.name = "customise"
-        self.description = "customise image during deployment"
-        self.summary = "customise image"
+    def __init__(self, parent, parameters):
+        super(Overlay, self).__init__(parent)
+        self.action = OverlayAction()
+        self.action.section = self.action_type
+        self.action.job = self.job
+        parent.add_action(self.action, parameters)
 
-    def run(self, connection, max_end_time, args=None):
-        connection = super(CustomisationAction, self).run(connection, max_end_time, args)
-        self.logger.debug("Customising image...")
-        # FIXME: implement
-        return connection
+    @classmethod
+    def accepts(cls, device, parameters):
+        if 'overlay' not in device['actions']['deploy']['methods']:
+            return False, "'overlay' not in the device configuration deploy methods"
+        if parameters['to'] != 'overlay':
+            return False, '"to" parameter is not "overlay"'
+        return True, 'accepted'
 
 
 # pylint: disable=too-many-instance-attributes
@@ -79,11 +85,12 @@ class OverlayAction(DeployAction):
     overlays are handled by TestDefinitionAction.
     """
 
+    name = "lava-overlay"
+    description = "add lava scripts during deployment for test shell use"
+    summary = "overlay the lava support scripts"
+
     def __init__(self):
         super(OverlayAction, self).__init__()
-        self.name = "lava-overlay"
-        self.description = "add lava scripts during deployment for test shell use"
-        self.summary = "overlay the lava support scripts"
         self.lava_test_dir = os.path.realpath(
             '%s/../../lava_test_shell' % os.path.dirname(__file__))
         self.scripts_to_copy = []
@@ -91,33 +98,41 @@ class OverlayAction(DeployAction):
         self.xmod = stat.S_IRWXU | stat.S_IXGRP | stat.S_IRGRP | stat.S_IXOTH | stat.S_IROTH
         self.target_mac = ''
         self.target_ip = ''
+        self.probe_ip = ''
+        self.probe_channel = ''
 
     def validate(self):
         super(OverlayAction, self).validate()
-        self.scripts_to_copy = glob.glob(os.path.join(self.lava_test_dir, 'lava-*'))
+        self.scripts_to_copy = sorted(glob.glob(os.path.join(self.lava_test_dir, 'lava-*')))
         # Distro-specific scripts override the generic ones
         if not self.test_needs_overlay(self.parameters):
             return
-        distro = self.parameters['deployment_data']['distro']
-        distro_support_dir = '%s/distro/%s' % (self.lava_test_dir, distro)
         lava_test_results_dir = self.parameters['deployment_data']['lava_test_results_dir']
         lava_test_results_dir = lava_test_results_dir % self.job.job_id
-        self.set_namespace_data(action='test', label='shared', key='lava_test_results_dir',
+        self.set_namespace_data(action='test', label='results', key='lava_test_results_dir',
                                 value=lava_test_results_dir)
         lava_test_sh_cmd = self.parameters['deployment_data']['lava_test_sh_cmd']
         self.set_namespace_data(action='test', label='shared', key='lava_test_sh_cmd',
                                 value=lava_test_sh_cmd)
-        for script in glob.glob(os.path.join(distro_support_dir, 'lava-*')):
-            self.scripts_to_copy.append(script)
+
+        # Add distro support scripts
+        distro = self.parameters['deployment_data']['distro']
+        distro_support_dir = '%s/distro/%s' % (self.lava_test_dir, distro)
+        self.scripts_to_copy += sorted(glob.glob(os.path.join(distro_support_dir,
+                                                              'lava-*')))
+
         if not self.scripts_to_copy:
             self.errors = "Unable to locate lava_test_shell support scripts."
-        if self.job.parameters.get('output_dir', None) is None:
-            self.errors = "Unable to use output directory."
         if 'parameters' in self.job.device:
             if 'interfaces' in self.job.device['parameters']:
                 if 'target' in self.job.device['parameters']['interfaces']:
                     self.target_mac = self.job.device['parameters']['interfaces']['target'].get('mac', '')
                     self.target_ip = self.job.device['parameters']['interfaces']['target'].get('ip', '')
+        for device in self.job.device.get('static_info', []):
+            if 'probe_channel' in device and 'probe_ip' in device:
+                self.probe_channel = device['probe_channel']
+                self.probe_ip = device['probe_ip']
+                break
 
     def populate(self, parameters):
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
@@ -145,6 +160,8 @@ class OverlayAction(DeployAction):
                 return connection
         self.set_namespace_data(action='test', label='shared', key='location', value=tmp_dir)
         lava_test_results_dir = self.get_namespace_data(action='test', label='results', key='lava_test_results_dir')
+        if not lava_test_results_dir:
+            raise LAVABug("Unable to identify top level lava test directory")
         shell = self.get_namespace_data(action='test', label='shared', key='lava_test_sh_cmd')
         self.logger.debug("[%s] Preparing overlay tarball in %s", namespace, tmp_dir)
         lava_path = os.path.abspath("%s/%s" % (tmp_dir, lava_test_results_dir))
@@ -158,13 +175,30 @@ class OverlayAction(DeployAction):
             with open(fname, 'r') as fin:
                 foutname = os.path.basename(fname)
                 output_file = '%s/bin/%s' % (lava_path, foutname)
-                self.logger.debug("Updating %s", output_file)
+                if "distro" in fname:
+                    distribution = os.path.basename(os.path.dirname(fname))
+                    self.logger.debug("Updating %s (%s)", output_file, distribution)
+                else:
+                    self.logger.debug("Creating %s", output_file)
                 with open(output_file, 'w') as fout:
                     fout.write("#!%s\n\n" % shell)
                     if foutname == 'lava-target-mac':
                         fout.write("TARGET_DEVICE_MAC='%s'\n" % self.target_mac)
                     if foutname == 'lava-target-ip':
                         fout.write("TARGET_DEVICE_IP='%s'\n" % self.target_ip)
+                    if foutname == 'lava-probe-ip':
+                        fout.write("PROBE_DEVICE_IP='%s'\n" % self.probe_ip)
+                    if foutname == 'lava-probe-channel':
+                        fout.write("PROBE_DEVICE_CHANNEL='%s'\n" % self.probe_channel)
+                    if foutname == 'lava-target-storage':
+                        fout.write('LAVA_STORAGE="\n')
+                        for method in self.job.device.get('storage_info', [{}]):
+                            for key, value in method.items():
+                                if key == 'yaml_line':
+                                    continue
+                                self.logger.debug("storage methods:\t%s\t%s", key, value)
+                                fout.write(r"\t%s\t%s\n" % (key, value))
+                        fout.write('"\n')
                     fout.write(fin.read())
                     os.fchmod(fout.fileno(), self.xmod)
 
@@ -183,12 +217,12 @@ class OverlayAction(DeployAction):
 
 class MultinodeOverlayAction(OverlayAction):
 
+    name = "lava-multinode-overlay"
+    description = "add lava scripts during deployment for multinode test shell use"
+    summary = "overlay the lava multinode scripts"
+
     def __init__(self):
         super(MultinodeOverlayAction, self).__init__()
-        self.name = "lava-multinode-overlay"
-        self.description = "add lava scripts during deployment for multinode test shell use"
-        self.summary = "overlay the lava multinode scripts"
-
         # Multinode-only
         self.lava_multi_node_test_dir = os.path.realpath(
             '%s/../../lava_test_shell/multi_node' % os.path.dirname(__file__))
@@ -275,15 +309,16 @@ class VlandOverlayAction(OverlayAction):
     """
     Adds data for vland interface locations, MAC addresses and vlan names
     """
+
+    name = "lava-vland-overlay"
+    description = "Populate specific vland scripts for tests to lookup vlan data."
+    summary = "Add files detailing vlan configuration."
+
     def __init__(self):
         super(VlandOverlayAction, self).__init__()
-        self.name = "lava-vland-overlay"
-        self.summary = "Add files detailing vlan configuration."
-        self.description = "Populate specific vland scripts for tests to lookup vlan data."
-
         # vland-only
         self.lava_vland_test_dir = os.path.realpath(
-            '%s/../../../lava_test_shell/vland' % os.path.dirname(__file__))
+            '%s/../../lava_test_shell/vland' % os.path.dirname(__file__))
         self.lava_vland_cache_file = '/tmp/lava_vland_cache.txt'
         self.params = {}
         self.sysfs = []
@@ -396,15 +431,12 @@ class CompressOverlay(Action):
     """
     Makes a tarball of the finished overlay and declares filename of the tarball
     """
-    def __init__(self):
-        super(CompressOverlay, self).__init__()
-        self.name = "compress-overlay"
-        self.summary = "Compress the lava overlay files"
-        self.description = "Create a lava overlay tarball and store alongside the job"
+    name = "compress-overlay"
+    description = "Create a lava overlay tarball and store alongside the job"
+    summary = "Compress the lava overlay files"
 
     def run(self, connection, max_end_time, args=None):
-        output = os.path.join(self.job.parameters['output_dir'],
-                              "overlay-%s.tar.gz" % self.level)
+        output = os.path.join(self.mkdtemp(), "overlay-%s.tar.gz" % self.level)
         location = self.get_namespace_data(action='test', label='shared', key='location')
         lava_test_results_dir = self.get_namespace_data(action='test', label='results', key='lava_test_results_dir')
         self.set_namespace_data(action='test', label='shared', key='output', value=output)
@@ -443,11 +475,13 @@ class SshAuthorize(Action):
     Used by secondary connections only.
     Primary connections need the keys set up by admins.
     """
+
+    name = "ssh-authorize"
+    description = 'include public key in overlay and authorize root user'
+    summary = 'add public key to authorized_keys'
+
     def __init__(self):
         super(SshAuthorize, self).__init__()
-        self.name = "ssh-authorize"
-        self.summary = 'add public key to authorized_keys'
-        self.description = 'include public key in overlay and authorize root user'
         self.active = False
         self.identity_file = None
 
@@ -513,12 +547,10 @@ class PersistentNFSOverlay(Action):
     so that it can be mounted later when the overlay is applied.
     """
 
-    def __init__(self):
-        super(PersistentNFSOverlay, self).__init__()
-        self.name = "persistent-nfs-overlay"
-        self.section = 'deploy'
-        self.summary = "add test overlay to NFS"
-        self.description = "unpack overlay into persistent NFS"
+    section = "deploy"
+    name = "persistent-nfs-overlay"
+    description = "unpack overlay into persistent NFS"
+    summary = "add test overlay to NFS"
 
     def validate(self):
         super(PersistentNFSOverlay, self).validate()
@@ -532,7 +564,7 @@ class PersistentNFSOverlay(Action):
             self.errors = "Unrecognised NFS URL: '%s'" % self.parameters['persistent_nfs']['address']
             return
         nfs_server, dirname = persist['address'].split(':')
-        self.errors = infrastructure_error('rpcinfo')
+        which('rpcinfo')
         self.errors = rpcinfo_nfs(nfs_server)
         self.set_namespace_data(action=self.name, label='nfs_address', key='nfsroot', value=dirname)
         self.set_namespace_data(action=self.name, label='nfs_address', key='serverip', value=nfs_server)
