@@ -24,14 +24,12 @@ import yaml
 import logging
 import unittest
 from lava_dispatcher.action import JobError
-from lava_dispatcher.utils.filesystem import mkdtemp
 from lava_dispatcher.device import NewDevice
 from lava_dispatcher.action import Timeout
 from lava_dispatcher.parser import JobParser
 from lava_dispatcher.actions.boot.ssh import SchrootAction
-from lava_dispatcher.utils.shell import infrastructure_error
 from lava_dispatcher.test.test_basic import Factory, StdoutTestCase
-from lava_dispatcher.test.utils import DummyLogger
+from lava_dispatcher.test.utils import DummyLogger, infrastructure_error
 from lava_dispatcher.utils.filesystem import check_ssh_identity_file
 from lava_dispatcher.protocols.multinode import MultinodeProtocol
 
@@ -43,23 +41,21 @@ class ConnectionFactory(Factory):  # pylint: disable=too-few-public-methods
     of any database objects.
     """
 
-    def create_ssh_job(self, filename, output_dir=None):  # pylint: disable=no-self-use
+    def create_ssh_job(self, filename):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/ssh-host-01.yaml'))
         kvm_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(kvm_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 0, None, dispatcher_config="",
-                               output_dir=output_dir)
+            job = parser.parse(sample_job_data, device, 0, None, dispatcher_config="")
             job.logger = DummyLogger()
         return job
 
-    def create_bbb_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
+    def create_bbb_job(self, filename):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         kvm_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(kvm_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, "",
-                               output_dir=output_dir)
+            job = parser.parse(sample_job_data, device, 4212, None, "")
             job.logger = DummyLogger()
         return job
 
@@ -69,8 +65,8 @@ class TestConnection(StdoutTestCase):  # pylint: disable=too-many-public-methods
     def setUp(self):
         super(TestConnection, self).setUp()
         factory = ConnectionFactory()
-        self.job = factory.create_ssh_job('sample_jobs/ssh-deploy.yaml', mkdtemp())
-        self.guest_job = factory.create_bbb_job('sample_jobs/bbb-ssh-guest.yaml', mkdtemp())
+        self.job = factory.create_ssh_job('sample_jobs/ssh-deploy.yaml')
+        self.guest_job = factory.create_bbb_job('sample_jobs/bbb-ssh-guest.yaml')
         logging.getLogger('dispatcher').addHandler(logging.NullHandler())
 
     @unittest.skipIf(infrastructure_error('schroot'), "schroot not installed")
@@ -191,7 +187,7 @@ class TestConnection(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_primary_ssh(self):
         factory = ConnectionFactory()
-        job = factory.create_ssh_job('sample_jobs/primary-ssh.yaml', mkdtemp())
+        job = factory.create_ssh_job('sample_jobs/primary-ssh.yaml')
         job.validate()
         overlay = [action for action in job.pipeline.actions if action.name == 'scp-overlay'][0]
         self.assertIsNotNone(overlay.parameters['deployment_data'])
@@ -233,6 +229,8 @@ class TestConnection(StdoutTestCase):  # pylint: disable=too-many-public-methods
         self.assertFalse(ssh.primary)
         self.assertIsNotNone(scp.scp)
         self.assertFalse(scp.primary)
+        autologin = [action for action in login.internal_pipeline.actions if action.name == 'auto-login-action'][0]
+        self.assertIsNone(autologin.params)
         self.assertIn('host_key', login.parameters['parameters'])
         self.assertIn('hostID', login.parameters['parameters'])
         self.assertIn(  # ipv4
@@ -284,6 +282,13 @@ class TestConnection(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
 class TestConsoleConnections(StdoutTestCase):
 
+    def setUp(self):
+        super(TestConsoleConnections, self).setUp()
+        factory = ConnectionFactory()
+        self.job = factory.create_ssh_job('sample_jobs/ssh-deploy.yaml')
+        self.guest_job = factory.create_bbb_job('sample_jobs/bbb-ssh-guest.yaml')
+        logging.getLogger('dispatcher').addHandler(logging.NullHandler())
+
     def test_device_commands(self):
         device_data = """
 commands:
@@ -309,17 +314,36 @@ commands:
             else:
                 self.assertEqual('uart0', hardware)
 
+    def test_connection_tags(self):
+        factory = ConnectionFactory()
+        job = factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
+        job.validate()
+        self.assertIsNotNone(job.device['commands']['connections'].items())
+        for hardware, value in job.device['commands']['connections'].items():
+            self.assertEqual('uart0', hardware)
+            self.assertIn('connect', value)
+            self.assertIn('tags', value)
+            self.assertEqual(['primary', 'telnet'], value['tags'])
+        boot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
+        connect = [action for action in boot.internal_pipeline.actions if action.name == 'connect-device'][0]
+        for hardware, value in connect.tag_dict.items():
+            self.assertEqual('uart0', hardware)
+            self.assertNotIn('connect', value)
+            self.assertNotIn('tags', value)
+            self.assertEqual(['primary', 'telnet'], value)
+        self.assertIn(connect.hardware, connect.tag_dict)
+        self.assertEqual(['primary', 'telnet'], connect.tag_dict[connect.hardware])
+
 
 class TestTimeouts(StdoutTestCase):
     """
     Test action and connection timeout parsing.
     """
 
-    def create_custom_job(self, data, output_dir='/tmp/'):  # pylint: disable=no-self-use
+    def create_custom_job(self, data):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         parser = JobParser()
-        job = parser.parse(data, device, 4212, None, "",
-                           output_dir=output_dir)
+        job = parser.parse(data, device, 4212, None, "")
         return job
 
     def test_action_timeout(self):
