@@ -42,8 +42,7 @@ from lava_dispatcher.parser import JobParser
 from lava_dispatcher.test.test_messages import FakeConnection
 from lava_dispatcher.utils.messages import LinuxKernelMessages
 from lava_dispatcher.test.test_defs import allow_missing_path, check_missing_path
-from lava_dispatcher.test.utils import DummyLogger
-from lava_dispatcher.utils.shell import infrastructure_error
+from lava_dispatcher.test.utils import DummyLogger, infrastructure_error
 from lava_dispatcher.utils.strings import substitute
 
 # pylint: disable=invalid-name
@@ -53,7 +52,7 @@ class TestBasicJob(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_basic_actions(self):
         factory = Factory()
-        job = factory.create_fake_qemu_job(mkdtemp())
+        job = factory.create_fake_qemu_job()
         if not job:
             return unittest.skip("not all deployments have been implemented")
         self.assertIsInstance(job, Job)
@@ -159,7 +158,7 @@ class TestKVMBasicDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
     def setUp(self):
         super(TestKVMBasicDeploy, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm.yaml')
 
     def test_deploy_job(self):
         self.assertEqual(self.job.pipeline.job, self.job)
@@ -187,7 +186,7 @@ class TestKVMBasicDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
         for action in self.job.pipeline.actions:
             self.assertIsNotNone(action.name)
             if isinstance(action, DeployAction):
-                overlay = action.pipeline.actions[3]
+                overlay = action.pipeline.actions[2]
         self.assertIsNotNone(overlay)
         # these tests require that lava-dispatcher itself is installed, not just running tests from a git clone
         self.assertTrue(os.path.exists(overlay.lava_test_dir))
@@ -227,7 +226,7 @@ class TestKVMQcow2Deploy(StdoutTestCase):  # pylint: disable=too-many-public-met
     def setUp(self):
         super(TestKVMQcow2Deploy, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-qcow2.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm-qcow2.yaml')
 
     def test_deploy_job(self):
         self.assertEqual(self.job.pipeline.job, self.job)
@@ -253,7 +252,7 @@ class TestKVMDownloadLocalDeploy(StdoutTestCase):  # pylint: disable=too-many-pu
     def setUp(self):
         super(TestKVMDownloadLocalDeploy, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-local.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm-local.yaml')
 
     def test_deploy_job(self):
         self.assertEqual(self.job.pipeline.job, self.job)
@@ -280,7 +279,7 @@ class TestKVMInlineTestDeploy(StdoutTestCase):  # pylint: disable=too-many-publi
     def setUp(self):
         super(TestKVMInlineTestDeploy, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-inline.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm-inline.yaml')
 
     def test_deploy_job(self):
         self.assertEqual(self.job.pipeline.job, self.job)
@@ -313,8 +312,7 @@ class TestKVMInlineTestDeploy(StdoutTestCase):  # pylint: disable=too-many-publi
                   """)
         self.assertIsInstance(device['actions']['boot']['methods']['qemu']['parameters']['extra'][1], int)
         parser = JobParser()
-        job = parser.parse(yaml.dump(job_data), device, 4212, None, "",
-                           output_dir='/tmp/')
+        job = parser.parse(yaml.dump(job_data), device, 4212, None, "")
         job.logger = DummyLogger()
         job.validate()
         boot_image = [action for action in job.pipeline.actions if action.name == 'boot-image-retry'][0]
@@ -334,9 +332,9 @@ class TestKVMInlineTestDeploy(StdoutTestCase):  # pylint: disable=too-many-publi
         inline_repo = None
         for action in self.job.pipeline.actions:
             if isinstance(action, DeployAction):
-                self.assertIsNotNone(action.internal_pipeline.actions[2])
-                overlay = action.pipeline.actions[2]
-                self.assertIsNotNone(overlay.internal_pipeline.actions[2])
+                self.assertIsNotNone(action.internal_pipeline.actions[1])
+                overlay = action.pipeline.actions[1]
+                self.assertIsNotNone(overlay.internal_pipeline.actions[1])
                 testdef = overlay.internal_pipeline.actions[2]
                 self.assertIsNotNone(testdef.internal_pipeline.actions[0])
                 inline_repo = testdef.internal_pipeline.actions[0]
@@ -380,7 +378,7 @@ class TestAutoLogin(StdoutTestCase):
     def setUp(self):
         super(TestAutoLogin, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-inline.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm-inline.yaml')
         self.job.logger = DummyLogger()
         self.max_end_time = time.time() + 30
 
@@ -407,6 +405,10 @@ class TestAutoLogin(StdoutTestCase):
 
         self.assertIn('lava-test: # ', conn.prompt_str)
         self.assertIn('root@debian:~#', conn.prompt_str)
+        conn.prompt_str = 'root@stretch:'
+        self.assertNotIn('lava-test: # ', conn.prompt_str)
+        self.assertNotIn('root@debian:~#', conn.prompt_str)
+        self.assertIn('root@stretch:', conn.prompt_str)
 
     def test_autologin_void_login_prompt(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -429,6 +431,7 @@ class TestAutoLogin(StdoutTestCase):
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': []})
+        autologinaction.parameters.update({'method': 'qemu'})
 
         with self.assertRaises((JobError, InfrastructureError)) as check:
             self.job.validate()
@@ -465,6 +468,8 @@ class TestAutoLogin(StdoutTestCase):
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': ['root@debian:~#']})
+        autologinaction.parameters.update({'method': 'qemu'})
+        autologinaction.validate()
 
         # initialise the first Connection object, a command line shell
         shell_connection = prepare_test_connection()
@@ -494,6 +499,8 @@ class TestAutoLogin(StdoutTestCase):
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': ['root@debian:~#']})
+        autologinaction.parameters.update({'method': 'qemu'})
+        autologinaction.validate()
 
         # initialise the first Connection object, a command line shell
         shell_connection = prepare_test_connection()
@@ -515,6 +522,8 @@ class TestAutoLogin(StdoutTestCase):
             'login_prompt': 'debian login:',
             'username': 'root'
         }})
+        autologinaction.parameters.update({'method': 'qemu'})
+        autologinaction.validate()
 
         # initialise the first Connection object, a command line shell
         shell_connection = prepare_test_connection()
@@ -533,7 +542,7 @@ class TestChecksum(StdoutTestCase):
     def setUp(self):
         super(TestChecksum, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-inline.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm-inline.yaml')
 
     def test_download_checksum_match_success(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -635,7 +644,7 @@ class TestChecksum(StdoutTestCase):
         bbb_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/bbb-ramdisk-nfs.yaml')
         with open(bbb_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, "", output_dir='/tmp/')
+            job = parser.parse(sample_job_data, device, 4212, None, "")
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
         download = [action for action in deploy.internal_pipeline.actions if action.name == 'download-retry'][0]
         helper = [action for action in download.internal_pipeline.actions if action.name == 'file-download'][0]
@@ -651,7 +660,7 @@ class TestKvmGuest(StdoutTestCase):  # pylint: disable=too-many-public-methods
     def setUp(self):
         super(TestKvmGuest, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-local.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm-local.yaml')
 
     def test_guest_size(self):
         self.assertIn('guest', self.job.device['actions']['deploy']['methods']['image']['parameters'])
@@ -663,7 +672,7 @@ class TestKvmUefi(StdoutTestCase):  # pylint: disable=too-many-public-methods
     def setUp(self):
         super(TestKvmUefi, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-uefi.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/kvm-uefi.yaml')
 
     @unittest.skipIf(infrastructure_error('qemu-system-x86_64'),
                      'qemu-system-x86_64 not installed')
@@ -695,8 +704,7 @@ class TestQemuNFS(StdoutTestCase):
         parser = JobParser()
         try:
             with open(kvm_yaml) as sample_job_data:
-                job = parser.parse(sample_job_data, device, 4212, None, "",
-                                   output_dir=mkdtemp())
+                job = parser.parse(sample_job_data, device, 4212, None, "")
         except NotImplementedError as exc:
             print(exc)
             # some deployments listed in basics.yaml are not implemented yet
@@ -755,7 +763,7 @@ class TestMonitor(StdoutTestCase):  # pylint: disable=too-many-public-methods
     def setUp(self):
         super(TestMonitor, self).setUp()
         factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/qemu-monitor.yaml', mkdtemp())
+        self.job = factory.create_kvm_job('sample_jobs/qemu-monitor.yaml')
 
     def test_qemu_monitor(self):
         self.assertIsNotNone(self.job)
