@@ -25,6 +25,7 @@ from lava_dispatcher.action import (
     Timeout,
     JobError,
     LAVABug,
+    InfrastructureError,
 )
 from lava_dispatcher.logical import (
     RetryAction,
@@ -32,7 +33,6 @@ from lava_dispatcher.logical import (
 )
 from lava_dispatcher.power import FinalizeAction
 from lava_dispatcher.job import Job
-from lava_dispatcher.utils.filesystem import mkdtemp
 from lava_dispatcher.test.test_basic import StdoutTestCase
 from lava_dispatcher.test.utils import DummyLogger
 
@@ -78,12 +78,13 @@ class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
         Isolated Action which can be used to generate artificial exceptions.
         """
 
+        name = "fake-action"
+        description = "fake, do not use outside unit tests"
+        summary = "fake action for unit tests"
+
         def __init__(self):
             super(TestAction.FakeAction, self).__init__()
             self.count = 1
-            self.name = "fake-action"
-            self.summary = "fake action for unit tests"
-            self.description = "fake, do not use outside unit tests"
 
         def run(self, connection, max_end_time, args=None):
             self.count += 1
@@ -94,13 +95,14 @@ class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
         Always fails, always triggers a diagnostic
         """
 
+        section = 'internal'
+        name = "trigger-action"
+        description = "fake, do not use outside unit tests"
+        summary = "fake trigger action for unit tests"
+
         def __init__(self):
             super(TestAction.FakeTriggerAction, self).__init__()
             self.count = 1
-            self.name = "trigger-action"
-            self.section = 'internal'
-            self.summary = "fake trigger action for unit tests"
-            self.description = "fake, do not use outside unit tests"
             self.parameters['namespace'] = 'common'
 
         def run(self, connection, max_end_time, args=None):
@@ -110,20 +112,16 @@ class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     class FakeRetryAction(RetryAction):
 
-        def __init__(self):
-            super(TestAction.FakeRetryAction, self).__init__()
-            self.name = "fake-retry-action"
-            self.summary = "fake retry action for unit tests"
-            self.description = "fake, do not use outside unit tests"
+        name = "fake-retry-action"
+        description = "fake, do not use outside unit tests"
+        summary = "fake retry action for unit tests"
 
     class InternalRetryAction(RetryAction):
 
-        def __init__(self):
-            super(TestAction.InternalRetryAction, self).__init__()
-            self.name = "internal-retry-action"
-            self.section = 'internal'
-            self.summary = "internal retry action for unit tests"
-            self.description = "internal, do not use outside unit tests"
+        section = 'internal'
+        name = "internal-retry-action"
+        description = "internal, do not use outside unit tests"
+        summary = "internal retry action for unit tests"
 
         def populate(self, parameters):
             self.internal_pipeline = Pipeline(parent=self, job=self.job)
@@ -131,12 +129,10 @@ class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     class CleanupRetryAction(RetryAction):
 
-        def __init__(self):
-            super(TestAction.CleanupRetryAction, self).__init__()
-            self.name = "internal-retry-action"
-            self.section = 'internal'
-            self.summary = "internal retry action for unit tests"
-            self.description = "internal, do not use outside unit tests"
+        section = 'internal'
+        name = "internal-retry-action"
+        description = "internal, do not use outside unit tests"
+        summary = "internal retry action for unit tests"
 
         def populate(self, parameters):
             self.internal_pipeline = Pipeline(parent=self, job=self.job)
@@ -158,7 +154,6 @@ class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
         super(TestAction, self).setUp()
         self.parameters = {
             "job_name": "fakejob",
-            'output_dir': mkdtemp(),
             "actions": [
                 {
                     'deploy': {
@@ -316,11 +311,9 @@ class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
         Isolated Action which can be used to generate artificial exceptions.
         """
 
-        def __init__(self):
-            super(TestTimeout.FakeAction, self).__init__()
-            self.name = "fake-action"
-            self.summary = "fake action for unit tests"
-            self.description = "fake, do not use outside unit tests"
+        name = "fake-action"
+        description = "fake, do not use outside unit tests"
+        summary = "fake action for unit tests"
 
         def populate(self, parameters):
             self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
@@ -337,11 +330,13 @@ class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
         """
         Isolated test action which passes
         """
+
+        name = "passing-action"
+        description = "fake action runs without calling adjuvant"
+        summary = "fake action without adjuvant"
+
         def __init__(self):
             super(TestTimeout.SafeAction, self).__init__()
-            self.name = "passing-action"
-            self.summary = "fake action without adjuvant"
-            self.description = "fake action runs without calling adjuvant"
             self.parameters['namespace'] = 'common'
 
         def run(self, connection, max_end_time, args=None):
@@ -354,11 +349,13 @@ class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
         """
         Isolated test action which times out the job itself
         """
+
+        name = "long-action"
+        description = "fake action"
+        summary = "fake action with overly long sleep"
+
         def __init__(self):
             super(TestTimeout.LongAction, self).__init__()
-            self.name = "long-action"
-            self.summary = "fake action with overly long sleep"
-            self.description = "fake action"
             self.parameters['namespace'] = 'common'
 
         def run(self, connection, max_end_time, args=None):
@@ -372,11 +369,10 @@ class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
         """
         Isolated test action which must not be run
         """
-        def __init__(self):
-            super(TestTimeout.SkippedAction, self).__init__()
-            self.name = "passing-action"
-            self.summary = "fake action without adjuvant"
-            self.description = "fake action runs without calling adjuvant"
+
+        name = "passing-action"
+        description = "fake action runs without calling adjuvant"
+        summary = "fake action without adjuvant"
 
         def run(self, connection, max_end_time, args=None):
             raise LAVABug("Fake action not meant to actually run - should have timed out")
@@ -386,11 +382,12 @@ class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
         Isolated Action which can be used to generate artificial exceptions.
         """
 
+        name = "fake-action"
+        description = "fake, do not use outside unit tests"
+        summary = "fake action for unit tests"
+
         def __init__(self):
             super(TestTimeout.FakeSafeAction, self).__init__()
-            self.name = "fake-action"
-            self.summary = "fake action for unit tests"
-            self.description = "fake, do not use outside unit tests"
             self.timeout.duration = 4
 
         def populate(self, parameters):
@@ -408,7 +405,6 @@ class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
         super(TestTimeout, self).setUp()
         self.parameters = {
             "job_name": "fakejob",
-            'output_dir': mkdtemp(),
             'timeouts': {
                 'job': {
                     'seconds': 3
@@ -454,6 +450,18 @@ class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
         with self.assertRaises(JobError):
             self.fakejob.run()
 
+    def test_action_timout_custom_exception(self):
+        """ Test custom timeout exception """
+        seconds = 2
+        pipeline = TestTimeout.FakePipeline(job=self.fakejob)
+        action = TestTimeout.FakeAction()
+        action.timeout = Timeout(action.name, duration=seconds, exception=InfrastructureError)
+        pipeline.add_action(action)
+        self.fakejob.pipeline = pipeline
+        self.fakejob.device = TestTimeout.FakeDevice()
+        with self.assertRaises(InfrastructureError):
+            self.fakejob.run()
+
     def test_action_complete(self):
         self.assertIsNotNone(self.fakejob.timeout)
         seconds = 2
