@@ -36,10 +36,9 @@ from lava_dispatcher.actions.deploy.tftp import TftpAction
 from lava_dispatcher.job import Job
 from lava_dispatcher.action import Pipeline, JobError
 from lava_dispatcher.test.test_basic import Factory, StdoutTestCase
-from lava_dispatcher.test.utils import DummyLogger
+from lava_dispatcher.test.utils import DummyLogger, infrastructure_error
 from lava_dispatcher.utils.network import dispatcher_ip
-from lava_dispatcher.utils.shell import infrastructure_error
-from lava_dispatcher.utils.filesystem import mkdtemp, tftpd_dir
+from lava_dispatcher.utils.filesystem import tftpd_dir
 from lava_dispatcher.utils.strings import substitute
 
 
@@ -49,23 +48,39 @@ class UBootFactory(Factory):  # pylint: disable=too-few-public-methods
     Factory objects are dispatcher based classes, independent
     of any database objects.
     """
-    def create_bbb_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
+    def create_bbb_job(self, filename):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         bbb_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(bbb_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, "",
-                               output_dir=output_dir)
+            job = parser.parse(sample_job_data, device, 4212, None, "")
             job.logger = DummyLogger()
         return job
 
-    def create_x15_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
+    def create_x15_job(self, filename):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/x15-01.yaml'))
         bbb_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(bbb_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, "",
-                               output_dir=output_dir)
+            job = parser.parse(sample_job_data, device, 4212, None, "")
+            job.logger = DummyLogger()
+        return job
+
+    def create_juno_job(self, filename):  # pylint: disable=no-self-use
+        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/juno-01.yaml'))
+        juno_yaml = os.path.join(os.path.dirname(__file__), filename)
+        with open(juno_yaml) as sample_job_data:
+            parser = JobParser()
+            job = parser.parse(sample_job_data, device, 4212, None, "")
+            job.logger = DummyLogger()
+        return job
+
+    def create_zcu102_job(self, filename):  # pylint: disable=no-self-use
+        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/xilinx-zcu102.yaml'))
+        zcu_yaml = os.path.join(os.path.dirname(__file__), filename)
+        with open(zcu_yaml) as sample_job_data:
+            parser = JobParser()
+            job = parser.parse(sample_job_data, device, 4212, None, "")
             job.logger = DummyLogger()
         return job
 
@@ -129,7 +144,7 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         params = job.device['actions']['deploy']['parameters']
         self.assertIn('mkimage_arch', params)
         boot_message = params.get('boot_message',
-                                  job.device.get_constant('boot-message'))
+                                  job.device.get_constant('kernel-start-message'))
         self.assertIsNotNone(boot_message)
         for action in job.pipeline.actions:
             action.validate()
@@ -154,6 +169,12 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         job.validate()
         description_ref = self.pipeline_reference('x15-uboot.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
+        deploy = [action for action in job.pipeline.actions if action.name == 'fastboot-deploy'][0]
+        enter = [action for action in deploy.internal_pipeline.actions if action.name == 'uboot-enter-fastboot'][0]
+        interrupt = [action for action in enter.internal_pipeline.actions if action.name == 'bootloader-interrupt'][0]
+        self.assertIsNotNone(interrupt.params)
+        self.assertNotEqual(interrupt.params, {})
+        self.assertEqual('u-boot', interrupt.method)
 
     def test_x15_uboot_nfs(self):  # pylint: disable=too-many-locals
         job = self.factory.create_x15_job('sample_jobs/x15-nfs.yaml')
@@ -166,6 +187,12 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         self.assertIn('compression', nfs.parameters['nfsrootfs'])
         self.assertEqual(nfs.parameters['nfsrootfs']['compression'], 'gz')
 
+    def test_juno_uboot_nfs(self):
+        job = self.factory.create_juno_job('sample_jobs/juno-uboot-nfs.yaml')
+        job.validate()
+        description_ref = self.pipeline_reference('juno-uboot-nfs.yaml', job=job)
+        self.assertEqual(description_ref, job.pipeline.describe(False))
+
     def test_overlay_action(self):  # pylint: disable=too-many-locals
         parameters = {
             'device_type': 'beaglebone-black',
@@ -173,7 +200,6 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
             'job_timeout': '15m',
             'action_timeout': '5m',
             'priority': 'medium',
-            'output_dir': mkdtemp(),
             'actions': {
                 'boot': {
                     'method': 'u-boot',
@@ -344,7 +370,7 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
                 uboot_retry = action
         names = [r_action.name for r_action in uboot_retry.internal_pipeline.actions]
         self.assertIn('reset-device', names)
-        self.assertIn('u-boot-interrupt', names)
+        self.assertIn('bootloader-interrupt', names)
         self.assertIn('expect-shell-connection', names)
         self.assertIn('bootloader-commands', names)
         for action in uboot_retry.internal_pipeline.actions:
@@ -361,8 +387,7 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         cubie = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/cubie1.yaml'))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/cubietruck-removable.yaml')
         sample_job_data = open(sample_job_file)
-        job = job_parser.parse(sample_job_data, cubie, 4212, None, "",
-                               output_dir='/tmp/')
+        job = job_parser.parse(sample_job_data, cubie, 4212, None, "")
         job.logger = DummyLogger()
         job.validate()
         sample_job_data.close()
@@ -410,6 +435,13 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         self.assertEqual(nfs.parameters['nfsrootfs']['prefix'], 'jessie/')
         self.assertEqual(nfs.param_key, 'nfsrootfs')
 
+    def test_zcu102(self):
+        job = self.factory.create_zcu102_job('sample_jobs/zcu102-ramdisk.yaml')
+        job.validate()
+        self.assertEqual(job.pipeline.errors, [])
+        description_ref = self.pipeline_reference('zcu102-ramdisk.yaml', job=job)
+        self.assertEqual(description_ref, job.pipeline.describe(False))
+
 
 class TestKernelConversion(StdoutTestCase):
 
@@ -430,7 +462,7 @@ class TestKernelConversion(StdoutTestCase):
 
     def test_zimage_bootz(self):
         self.deploy_block['kernel']['type'] = 'zimage'
-        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "")
         job.logger = DummyLogger()
         job.validate()
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
@@ -443,7 +475,7 @@ class TestKernelConversion(StdoutTestCase):
 
     def test_image(self):
         self.deploy_block['kernel']['type'] = 'image'
-        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "")
         job.logger = DummyLogger()
         job.validate()
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
@@ -457,7 +489,7 @@ class TestKernelConversion(StdoutTestCase):
 
     def test_uimage(self):
         self.deploy_block['kernel']['type'] = 'uimage'
-        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "")
         job.logger = DummyLogger()
         job.validate()
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
@@ -472,7 +504,7 @@ class TestKernelConversion(StdoutTestCase):
         # drop bootz from the device for this part of the test
         del self.device['parameters']['bootz']
         self.deploy_block['kernel']['type'] = 'zimage'
-        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "")
         job.logger = DummyLogger()
         job.validate()
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
@@ -487,7 +519,7 @@ class TestKernelConversion(StdoutTestCase):
         # uimage in boot type
         del self.deploy_block['kernel']['type']
         self.boot_block['type'] = 'bootm'
-        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "")
         job.logger = DummyLogger()
         job.validate()
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
